# Classic Histogram Equalization 16bit
# Ver. 0.1 (2024_04)
# (c) 2024, Ohnishi, Yasuo
from ij import IJ
from ij import WindowManager as WM
from ij.io import OpenDialog
from loci.plugins import BF
from ij.plugin import ChannelSplitter
from ij.plugin import Duplicator
from ij.process import ImageConverter
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')
from java.awt import *
import jarray
#----------------
def equalize(imp, Mode):
    if Mode =="32-bit":
        IJ.showMessage("Equalization of 32-bit images not supported.")
        return
    imp2 = Duplicator().run(imp)
    ip2 = imp2.getProcessor()
    histogram = ip2.getHistogram()
    equalize_ip(ip2, histogram, Mode)
    imp2.setProcessor(ip2)
    imp2.getProcessor().resetMinAndMax()
    return imp2
#----------
def equalize_ip(ip, histogram, Mode):
    ip.resetRoi()
    if Mode =="16-bit":
        max = 65535
        rangeHist = 65535
    else: #bytes
        max = 255
        rangeHist = 255
        
    sum = histogram[0]
    for i in range(max):
        sum += 2 * histogram[i]
    sum += histogram[max]
    scale = float(rangeHist) / float(sum)
    lut = jarray.array([0] * (rangeHist + 1),'i')
    sum = histogram[0]
    for i in range(max):
        delta = histogram[i]
        sum += delta
        lut[i] = int(round(sum * scale)) #
        sum += delta

    lut[max] = max
    ip.applyTable(lut)
    return ip
 
def bitDepthConvert(imp, Mode):
    imp1 = Duplicator().run(imp)
    ImageConverter(imp1).convertToGray16()
    if Mode == "8-bit":
        imp1.getProcessor().multiply(257)
    return imp1

#
window = WM.getActiveWindow()
if not window:
    od = OpenDialog("Choose a file")
    folder = od.getDirectory()
    filename = od.getFileName()
    if filename:
        path = folder + filename
        IJ.log(path)
        IJ.log(folder)
        IJ.log(filename)
# Always Open file with Bio-format Plugin
        imps = BF.openImagePlus(folder + filename) 
        imp0 = imps[0] #imp0: Original Image 
        imp0.show()
    else:
        IJ.showMessage("File is not selected.")
else: # target image is already open
    imp0 = IJ.getImage()
    imagename = imp0.getTitle()
    filename = imagename
    try:
        folder = imp0.getOriginalFileInfo().directory
    except:
        folder = IJ.getDirectory("Output_directory")

bitdepth = imp0.getBitDepth()
stackSize =imp0.getStackSize()
if bitdepth == 8 or bitdepth == 24:
    Mode = "8-bit"
elif bitdepth ==16:
    Mode = "16-bit"
elif bitdepth ==32:
    Mode = "32-bit"
else:
    Mode = "16-bit"
if stackSize > 1:
    chName = ['R', 'G', 'B'] # Channel name
    impsCh1 = []
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in range(stackSize):
        if Mode != "16-bit":
            impsCh1.append(equalize(bitDepthConvert(impsCh[i], Mode), "16-bit"))
        else:
            impsCh1.append(equalize(impsCh[i], Mode))
        impsCh1[i].setTitle(chName[i])
        
        impsCh1[i].show()
    imp1 = IJ.run("Merge Channels...", "c1=R c2=G c3=B create keep")
else: #"Mono channel"
    if Mode != "16-bit":
        imp1 =equalize(bitDepthConvert(imp0, Mode), "16-bit")
    else:
        imp1 = equalize(imp0, Mode)
IJ.saveAsTiff(imp1, folder + filename +"_equalized_16.tif")
IJ.showMessage("Process finished.")
if stackSize > 1:
    for i in range(stackSize):
        impsCh[i].close() 
        impsCh1[i].hide() 
        impsCh1[i].close() 
imp0.hide()
imp0.close()
